@@ -11,7 +11,7 @@
 #include <pthread.h>
 #include <string.h>
 #include <ccs.h>
-#include <corosync/engine/logsys.h>
+#include <liblogthread.h>
 #include <sched.h>
 #include <sys/mman.h>
 #include "disk.h"
@@ -20,7 +20,7 @@
 static pthread_mutex_t sc_lock = PTHREAD_MUTEX_INITIALIZER;
 static int _score = 0, _maxscore = 0, _score_thread_running = 0;
 static pthread_t score_thread = (pthread_t)0;
-void set_priority(int, int);
+extern void set_priority(int, int);
 
 struct h_arg {
 	struct h_data *h;
@@ -116,8 +116,8 @@ fork_heuristic(struct h_data *h)
 	munlockall();
 	restore_signals();
 
-	argv[0] = "/bin/sh";
-	argv[1] = "-c";
+	argv[0] = strdup("/bin/sh");
+	argv[1] = strdup("-c");
 	argv[2] = h->program;
 	argv[3] = NULL;
 
@@ -125,7 +125,10 @@ fork_heuristic(struct h_data *h)
 
 	execv("/bin/sh", argv);
 
-	log_printf(LOG_ERR, "Execv failed\n");
+	free(argv[0]);
+	free(argv[1]);
+
+	logt_print(LOG_ERR, "Execv failed\n");
 	return 0;
 }
 
@@ -141,7 +144,6 @@ total_score(struct h_data *h, int max, int *score, int *maxscore)
 	*score = 0;
 	*maxscore = 0;
 	
-	log_printf(LOG_DEBUG, "max = %d\n", max);
 	/* Allow operation w/o any heuristics */
 	if (!max) {
 		*score = *maxscore = 1;
@@ -190,7 +192,7 @@ check_heuristic(struct h_data *h, int block)
 	/* Returned 0 and was not killed */
 	if (!h->available) {
 		h->available = 1;
-		log_printf(LOG_INFO, "Heuristic: '%s' UP\n", h->program);
+		logt_print(LOG_INFO, "Heuristic: '%s' UP\n", h->program);
 	}
 	h->misses = 0;
 	return 0;
@@ -199,12 +201,12 @@ miss:
 	if (h->available) {
 		h->misses++;
 		if (h->misses >= h->tko) {
-			log_printf(LOG_INFO,
+			logt_print(LOG_INFO,
 				"Heuristic: '%s' DOWN (%d/%d)\n",
 				h->program, h->misses, h->tko);
 			h->available = 0;
 		} else {
-			log_printf(LOG_DEBUG,
+			logt_print(LOG_DEBUG,
 				"Heuristic: '%s' missed (%d/%d)\n",
 				h->program, h->misses, h->tko);
 		}
@@ -303,13 +305,13 @@ configure_heuristics(int ccsfd, struct h_data *h, int max)
 				h[x].tko = 1;
 		}
 
-		log_printf(LOG_DEBUG,
+		logt_print(LOG_DEBUG,
 		       "Heuristic: '%s' score=%d interval=%d tko=%d\n",
 		       h[x].program, h[x].score, h[x].interval, h[x].tko);
 
 	} while (++x < max);
 
-	log_printf(LOG_DEBUG, "%d heuristics loaded\n", x);
+	logt_print(LOG_DEBUG, "%d heuristics loaded\n", x);
 		
 	return x;
 }
@@ -371,7 +373,7 @@ score_thread_main(void *arg)
 
 	free(args->h);
 	free(args);
-	log_printf(LOG_INFO, "Score thread going away\n");
+	logt_print(LOG_INFO, "Score thread going away\n");
 	return (NULL);
 }
 
