@@ -1,14 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) 2008 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
 #include <sys/types.h>
 #include <stdlib.h>
 #include <string.h>
@@ -25,23 +14,25 @@
 #include <corosync/engine/config.h>
 
 /* These are defaults. they can be overridden with environment variables
- *  LDAP_URL & LDAP_BASEDN
+ *  COROSYNC_LDAP_URL & COROSYNC_LDAP_BASEDN
  */
 #define DEFAULT_LDAP_URL "ldap:///"
 #define DEFAULT_LDAP_BASEDN "dc=chrissie,dc=net"
 
-static int ldap_readconfig(struct objdb_iface_ver0 *objdb, char **error_string);
+static int ldap_readconfig(struct objdb_iface_ver0 *objdb, const char **error_string);
+static int ldap_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, const char **error_string);
 static int init_config(struct objdb_iface_ver0 *objdb);
 static char error_reason[1024];
-static char *ldap_url = DEFAULT_LDAP_URL;
-static char *ldap_basedn = DEFAULT_LDAP_BASEDN;
+static const char *ldap_url = DEFAULT_LDAP_URL;
+static const char *ldap_basedn = DEFAULT_LDAP_BASEDN;
 
 /*
  * Exports the interface for the service
  */
 
 static struct config_iface_ver0 ldapconfig_iface_ver0 = {
-	.config_readconfig        = ldap_readconfig
+	.config_readconfig        = ldap_readconfig,
+	.config_reloadconfig        = ldap_reloadconfig
 };
 
 static struct lcr_iface ifaces_ver0[2] = {
@@ -70,7 +61,7 @@ __attribute__ ((constructor)) static void ldap_comp_register(void) {
 	lcr_component_register(&ldap_comp_ver0);
 };
 
-static int ldap_readconfig(struct objdb_iface_ver0 *objdb, char **error_string)
+static int ldap_readconfig(struct objdb_iface_ver0 *objdb, const char **error_string)
 {
 	int ret;
 
@@ -83,6 +74,11 @@ static int ldap_readconfig(struct objdb_iface_ver0 *objdb, char **error_string)
 	return ret;
 }
 
+static int ldap_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, const char **error_string)
+{
+	return ldap_readconfig(objdb, error_string);
+}
+
 /*
  * Convert hyphens to underscores in all attribute names
  */
@@ -110,14 +106,14 @@ static void convert_dn_underscores(LDAPDN dn)
  * Return the parent object of a DN.
  * Actually, this returns the LAST parent with that name. which should (!) be correct.
  */
-static unsigned int find_parent(struct objdb_iface_ver0 *objdb, LDAPDN dn, int startdn, char *parent)
+static hdb_handle_t find_parent(struct objdb_iface_ver0 *objdb, LDAPDN dn, int startdn, const char *parent)
 {
 	int i=startdn;
 	int gotstart=0;
 	int start=0, end=startdn;
-	unsigned int parent_handle = OBJECT_PARENT_HANDLE;
-	unsigned int object_handle=0;
-	unsigned int find_handle;
+	hdb_handle_t parent_handle = OBJECT_PARENT_HANDLE;
+	hdb_handle_t object_handle=0;
+	hdb_handle_t find_handle;
 
 	/*
 	 * Find the start and end positions first.
@@ -149,15 +145,15 @@ static unsigned int find_parent(struct objdb_iface_ver0 *objdb, LDAPDN dn, int s
 
 
 
-static int read_config_for(LDAP *ld, struct objdb_iface_ver0 *objdb, unsigned int parent,
-			   char *object, char *sub_dn, int always_create)
+static int read_config_for(LDAP *ld, struct objdb_iface_ver0 *objdb, hdb_handle_t parent,
+			   const char *object, const char *sub_dn, int always_create)
 {
 	char search_dn[4096];
 	int rc;
 	char *dn;
 	LDAPMessage *result, *e;
-	unsigned int parent_handle = OBJECT_PARENT_HANDLE;
-	unsigned int object_handle;
+	hdb_handle_t parent_handle = OBJECT_PARENT_HANDLE;
+	hdb_handle_t object_handle;
 
 	sprintf(search_dn, "%s,%s", sub_dn, ldap_basedn);
 
@@ -199,6 +195,9 @@ static int read_config_for(LDAP *ld, struct objdb_iface_ver0 *objdb, unsigned in
 						     parsed_dn[0][0][0].la_value.bv_len);
 			}
 			else {
+ 			        /* Remove redundant empty parent. */
+			        objdb->object_destroy(object_handle);
+
 				parent_handle = find_parent(objdb, parsed_dn, 2, object);
 				/* Create a new object with the same name as the current one */
 				objdb->object_create(parent_handle, &object_handle, parsed_dn[1][0][0].la_value.bv_val,
@@ -268,7 +267,7 @@ static int init_config(struct objdb_iface_ver0 *objdb)
 
 	/* Connect to the LDAP server */
 	if (ldap_initialize(&ld, ldap_url)) {
-		sprintf(error_reason, "ldap_simple_bind failed: %s\n", strerror(errno));
+		sprintf(error_reason, "ldap_initialize failed: %s\n", strerror(errno));
 		return -1;
 	}
 	version = LDAP_VERSION3;
@@ -283,7 +282,7 @@ static int init_config(struct objdb_iface_ver0 *objdb)
 		return -1;
 	}
 
-	rc = read_config_for(ld, objdb, OBJECT_PARENT_HANDLE, "cluster", "cn=cluster", 1);
+	rc = read_config_for(ld, objdb, OBJECT_PARENT_HANDLE, "cluster", "name=cluster", 1);
 
 	ldap_unbind(ld);
 	return 0;
