@@ -25,7 +25,7 @@ BUILD_DATE="March, 2008"
 #END_VERSION_GENERATION
 
 def get_power_status(conn, options):
-	result = ""
+	exp_result = 0
 	outlets = {}
 	try:
 		conn.send("1\r\n")
@@ -67,16 +67,17 @@ def get_power_status(conn, options):
 		else:
 			conn.send(options["-s"]+"\r\n")
 			
-		while 1 == conn.log_expect(options, [ options["-c"],  "Press <ENTER>" ], SHELL_TIMEOUT):
-			result += conn.before
+		while True:
+			exp_result = conn.log_expect(options, [ options["-c"],  "Press <ENTER>" ], SHELL_TIMEOUT)
 			lines = conn.before.split("\n");
-			show_re = re.compile('^\s*(\d+)- (.*?)\s+(ON|OFF)\s*$')
+			show_re = re.compile('^\s*(\d+)- (.*?)\s+(ON|OFF)\s*')
 			for x in lines:
 				res = show_re.search(x)
 				if (res != None):
 					outlets[res.group(1)] = (res.group(2), res.group(3))
 			conn.send("\r\n")
-		result += conn.before
+			if exp_result == 0:
+				break
 		conn.send(chr(03))		
 		conn.log_expect(options, "- Logout", SHELL_TIMEOUT)
 		conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
@@ -88,8 +89,11 @@ def get_power_status(conn, options):
 	if ["list", "monitor"].count(options["-o"]) == 1:
 		return outlets
 	else:
-		status = re.compile("\s*"+options["-n"]+"-.*(ON|OFF)", re.IGNORECASE).search(result).group(1)
-		return status.lower().strip()
+		try:
+			(_, status) = outlets[options["-n"]]
+			return status.lower().strip()
+		except KeyError:
+			fail(EC_STATUS)
 
 def set_power_status(conn, options):
 	action = {
@@ -102,7 +106,8 @@ def set_power_status(conn, options):
 		conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
 
 		version = 0
-		admin = 0
+		admin2 = 0
+		admin3 = 0
 		switch = 0
 
 		if (None != re.compile('.* MasterSwitch plus.*', re.IGNORECASE | re.S).match(conn.before)):
@@ -125,19 +130,23 @@ def set_power_status(conn, options):
 			version = 3
 
 		if (None == re.compile('.*Outlet Control/Configuration.*', re.IGNORECASE | re.S).match(conn.before)):
-			admin = 0
+			admin2 = 0
 		else:
-			admin = 1
+			admin2 = 1
 
 		if switch == 0:
 			if version == 2:
-				if admin == 0:
+				if admin2 == 0:
 					conn.send("2\r\n")
 				else:
 					conn.send("3\r\n")
 			else:
 				conn.send("2\r\n")
 				conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
+				if (None == re.compile('.*2- Outlet Restriction.*', re.IGNORECASE | re.S).match(conn.before)):
+					admin3 = 0
+				else:
+					admin3 = 1
 				conn.send("1\r\n")
 		else:
 			conn.send(options["-s"] + "\r\n")
@@ -148,10 +157,10 @@ def set_power_status(conn, options):
 		conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
 
 		if switch == 0:
-			if admin == 1:
+			if admin2 == 1:
 				conn.send("1\r\n")
 				conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
-			if version == 3:
+			if admin3 == 1:
 				conn.send("1\r\n")
 				conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
 		else:
@@ -175,7 +184,10 @@ def set_power_status(conn, options):
 def main():
 	device_opt = [  "help", "version", "agent", "quiet", "verbose", "debug",
 			"action", "ipaddr", "login", "passwd", "passwd_script",
-			"secure", "port", "switch", "test", "separator" ]
+			"secure", "port", "switch", "test", "separator",
+			"inet4_only", "inet6_only", "ipport" ]
+
+	atexit.register(atexit_handler)
 
 	options = check_input(device_opt, process_input(device_opt))
 
@@ -187,6 +199,8 @@ def main():
 	if 0 == options.has_key("-c"):
 		options["-c"] = "\n>"
 
+	show_docs(options)
+
 	## Support for -n [switch]:[plug] notation that was used before
 	if (options.has_key("-n") == 1) and (-1 != options["-n"].find(":")):
 		(switch, plug) = options["-n"].split(":", 1)
@@ -211,6 +225,8 @@ def main():
 		conn.close()
 	except exceptions.OSError:
 		pass
+	except pexpect.ExceptionPexpect:
+		pass
 
 if __name__ == "__main__":
 	main()
