@@ -63,7 +63,8 @@ extern int daemon_debug_opt;
 extern int daemon_quit;
 extern int cluster_down;
 extern struct list_head domains;
-extern int cman_quorate;
+extern int cluster_quorate;
+extern uint64_t quorate_time;
 extern int our_nodeid;
 extern char our_name[MAX_NODENAME_LEN+1];
 extern char daemon_debug_buf[256];
@@ -122,6 +123,7 @@ struct change {
 	int state; /* CGST_ */
 	int we_joined;
 	uint32_t seq; /* just used as a reference when debugging */
+	uint64_t create_time;
 };
 
 #define VIC_DONE_AGENT		1
@@ -138,6 +140,8 @@ struct node_history {
 	uint64_t fail_time;
 	uint64_t fence_time;
 	uint64_t fence_external_time;
+	uint64_t cluster_add_time;
+	uint64_t cluster_remove_time;
 	int fence_external_node;
 	int fence_master;
 	int fence_how; /* VIC_DONE_ */
@@ -147,6 +151,7 @@ struct node {
 	struct list_head 	list;
 	int			nodeid;
 	int			init_victim;
+	int			local_victim_done;
 	char 			name[MAX_NODENAME_LEN+1];
 };
 
@@ -165,6 +170,7 @@ struct fd {
 	struct list_head	changes;
 	struct list_head	node_history;
 	int			init_complete;
+	int			local_init_complete;
 
 	/* general domain membership */
 
@@ -198,9 +204,9 @@ int read_ccs(struct fd *fd);
 
 /* cpg.c */
 
-void process_cpg(int ci);
-int setup_cpg(void);
-void close_cpg(void);
+void process_cpg_daemon(int ci);
+int setup_cpg_daemon(void);
+void close_cpg_daemon(void);
 int set_protocol(void);
 void free_cg(struct change *cg);
 void node_history_init(struct fd *fd, int nodeid);
@@ -216,7 +222,9 @@ int set_node_info(struct fd *fd, int nodeid, struct fenced_node *node);
 int set_domain_info(struct fd *fd, struct fenced_domain *domain);
 int set_domain_nodes(struct fd *fd, int option, int *node_count,
 		     struct fenced_node **nodes);
-int in_daemon_member_list(int nodeid);
+int is_clean_daemon_member(int nodeid);
+void node_history_cluster_add(int nodeid);
+void node_history_cluster_remove(int nodeid);
 
 /* group.c */
 
@@ -243,10 +251,10 @@ void cluster_dead(int ci);
 
 /* member_cman.c */
 
-void process_cman(int ci);
-int setup_cman(void);
-void close_cman(void);
-int is_cman_member(int nodeid);
+void process_cluster(int ci);
+int setup_cluster(void);
+void close_cluster(void);
+int is_cluster_member_reread(int nodeid);
 char *nodeid_to_name(int nodeid);
 int name_to_nodeid(char *name);
 struct node *get_new_node(struct fd *fd, int nodeid);
