@@ -173,7 +173,7 @@ static void _add_first_victims(struct fd *fd)
 		log_debug("first complete list empty warning");
 
 	list_for_each_entry_safe(prev_node, safe, &fd->complete, list) {
-		if (!is_cman_member(prev_node->nodeid)) {
+		if (!is_cluster_member_reread(prev_node->nodeid)) {
 			list_del(&prev_node->list);
 			list_add(&prev_node->list, &fd->victims);
 			log_debug("add first victim %s", prev_node->name);
@@ -381,8 +381,10 @@ int fd_join_group(struct fd *fd)
 		log_error("group_join error %d", rv);
 		list_del(&fd->list);
 		free(fd);
+		return rv;
 	}
-	return rv;
+	set_cman_dirty();
+	return 0;
 }
 
 int fd_leave_group(struct fd *fd)
@@ -444,31 +446,44 @@ int set_domain_nodes_group(struct fd *fd, int option, int *node_count,
 	return 0;
 }
 
-void set_group_mode(void)
+int set_group_mode(void)
 {
-	int i = 0, rv, version;
+	int i = 0, rv, version, limit;
 
 	while (1) {
 		rv = group_get_version(&version);
-		if (rv) {
-			log_error("group_get_version error %d", rv);
-			break;
-		} else if (version == GROUP_LIBGROUP) {
-			group_mode = GROUP_LIBGROUP;
-			break;
-		} else if (version == GROUP_LIBCPG) {
-			group_mode = GROUP_LIBCPG;
-			break;
-		} else if (version != -EAGAIN) {
-			log_error("group_get_version result %d", version);
-			break;
+
+		if (rv || version < 0) {
+			/* we expect to get version of -EAGAIN while groupd
+			   is detecting the mode of everyone; don't retry
+			   as long if we're not getting anything back from
+			   groupd */
+
+			log_debug("set_group_mode get_version %d ver %d",
+				  rv, version);
+
+			limit = (version == -EAGAIN) ? 30 : 5;
+
+			if (i++ > limit) {
+				log_error("cannot get groupd compatibility "
+					  "mode rv %d ver %d", rv, version);
+				return -1;
+			}
+			sleep(1);
+			continue;
 		}
 
-		if (i++ > 10) {
-			log_error("no version from groupd");
-			break;
+
+		if (version == GROUP_LIBGROUP) {
+			group_mode = GROUP_LIBGROUP;
+			return 0;
+		} else if (version == GROUP_LIBCPG) {
+			group_mode = GROUP_LIBCPG;
+			return 0;
+		} else {
+			log_error("set_group_mode invalid ver %d", version);
+			return -1;
 		}
-		sleep(1);
 	}
 }
 
