@@ -1,13 +1,15 @@
 #ifndef __HEXVIEW_DOT_H__
 #define __HEXVIEW_DOT_H__
 
-#include "linux_endian.h"
 #include <sys/types.h>
 #include <inttypes.h>
 #include <limits.h>
 #include <linux/gfs2_ondisk.h>
 #include <string.h>
 
+#include "libgfs2.h"
+#include "copyright.cf"
+
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -15,17 +17,6 @@
 #define FALSE 0
 #endif
 
-/*  Extern Macro  */
-
-#ifndef EXTERN
-#define EXTERN extern
-#define INIT(X)
-#else
-#undef EXTERN
-#define EXTERN
-#define INIT(X) =X 
-#endif
-
 #define DMODES 3
 enum dsp_mode { HEX_MODE = 0, GFS2_MODE = 1, EXTENDED_MODE = 2 };
 #define BLOCK_STACK_SIZE 256
@@ -51,42 +42,45 @@ enum dsp_mode { HEX_MODE = 0, GFS2_MODE = 1, EXTENDED_MODE = 2 };
 #define GFS_LOG_DESC_Q          (402)    /* quota */
 #define GFS_LOG_DESC_LAST       (500)    /* final in a logged transaction */
 
-EXTERN char *prog_name;
-EXTERN uint64_t block INIT(0);
-EXTERN int blockhist INIT(0);
-EXTERN int edit_mode INIT(0);
-EXTERN int line;
-EXTERN char edit_fmt[80];
-EXTERN char estring[1024]; /* edit string */
-EXTERN uint64_t dev_offset INIT(0);
-EXTERN uint64_t max_block INIT(0);
-EXTERN char *buf INIT(NULL);
-EXTERN int termlines INIT(30);
-EXTERN int termcols INIT(80);
-EXTERN int insert INIT(0);
-EXTERN const char *termtype;
-EXTERN int line INIT(1);
-EXTERN int struct_len INIT(0);
-EXTERN unsigned int offset;
-EXTERN int edit_row[DMODES], edit_col[DMODES], print_entry_ndx;
-EXTERN int start_row[DMODES], end_row[DMODES], lines_per_row[DMODES];
-EXTERN int edit_size[DMODES], last_entry_onscreen[DMODES];
-EXTERN char edit_fmt[80];
-EXTERN struct gfs2_sbd sbd;
-EXTERN struct gfs_sb *sbd1;
-EXTERN struct gfs2_inum gfs1_quota_di;   /* kludge because gfs2 sb too small */
-EXTERN struct gfs2_inum gfs1_license_di; /* kludge because gfs2 sb too small */
-EXTERN struct gfs2_dinode di;
-EXTERN int screen_chunk_size INIT(512); /* how much of the 4K can fit on screen */
-EXTERN int gfs2_struct_type;
-EXTERN uint64_t block_in_mem INIT(-1);
-EXTERN char device[NAME_MAX];
-EXTERN int identify INIT(FALSE);
-EXTERN int color_scheme INIT(0);
-EXTERN WINDOW *wind;
-EXTERN int gfs1 INIT(0);
-EXTERN int editing INIT(0);
-EXTERN uint64_t temp_blk;
+extern uint64_t block;
+extern int blockhist;
+extern int edit_mode;
+extern int line;
+extern char edit_fmt[80];
+extern char estring[1024]; /* edit string */
+extern char efield[64];
+extern uint64_t dev_offset;
+extern uint64_t max_block;
+extern char *buf;
+extern int termlines;
+extern int termcols;
+extern int insert;
+extern const char *termtype;
+extern int line;
+extern int struct_len;
+extern unsigned int offset;
+extern int edit_row[DMODES], edit_col[DMODES], print_entry_ndx;
+extern int start_row[DMODES], end_row[DMODES], lines_per_row[DMODES];
+extern int edit_size[DMODES], last_entry_onscreen[DMODES];
+extern char edit_fmt[80];
+extern struct gfs2_sbd sbd;
+extern struct gfs_sb *sbd1;
+extern struct gfs2_inum gfs1_quota_di;   /* kludge because gfs2 sb too small */
+extern struct gfs2_inum gfs1_license_di; /* kludge because gfs2 sb too small */
+extern struct gfs2_dinode di;
+extern int screen_chunk_size; /* how much of the 4K can fit on screen */
+extern int gfs2_struct_type;
+extern uint64_t block_in_mem;
+extern char device[NAME_MAX];
+extern int identify;
+extern int color_scheme;
+extern WINDOW *wind;
+extern int gfs1;
+extern int editing;
+extern uint64_t temp_blk;
+extern uint64_t starting_blk;
+extern const char *block_type_str[15];
+extern int dsplines;
 
 struct gfs_jindex {
         uint64_t ji_addr;       /* starting block of the journal */
@@ -153,15 +147,21 @@ struct gfs_rgrp {
 	char rg_reserved[64];
 };
 
-EXTERN int block_is_jindex(void);
-EXTERN int block_is_rindex(void);
-EXTERN int block_is_inum_file(void);
-EXTERN int block_is_statfs_file(void);
-EXTERN int block_is_quota_file(void);
-EXTERN int display_block_type(const char *lpBuffer, int from_restore);
-EXTERN void gfs_jindex_in(struct gfs_jindex *jindex, char *buf);
-EXTERN void gfs_log_header_in(struct gfs_log_header *head, char *buf);
-EXTERN void gfs_log_header_print(struct gfs_log_header *lh);
+extern int block_is_jindex(void);
+extern int block_is_rindex(void);
+extern int block_is_inum_file(void);
+extern int block_is_statfs_file(void);
+extern int block_is_quota_file(void);
+extern int block_is_per_node(void);
+extern int block_is_in_per_node(void);
+extern int display_block_type(const char *lpBuffer, int from_restore);
+extern void gfs_jindex_in(struct gfs_jindex *jindex, char *buf);
+extern void gfs_log_header_in(struct gfs_log_header *head, char *buf);
+extern void gfs_log_header_print(struct gfs_log_header *lh);
+extern void gfs_dinode_in(struct gfs_dinode *di, char *buf);
+extern void savemeta(char *out_fn, int saveoption);
+extern void restoremeta(const char *in_fn, const char *out_device, int printblocksonly);
+extern uint64_t masterblock(const char *fn);
 
 struct gfs2_dirents {
 	uint64_t block;
@@ -171,8 +171,10 @@ struct gfs2_dirents {
 
 struct indirect_info {
 	int is_dir;
+	int height;
 	uint64_t block;
 	uint32_t dirents;
+	struct metapath mp;
 	struct gfs2_dirents dirent[64];
 };
 
@@ -180,12 +182,6 @@ struct iinfo {
 	struct indirect_info ii[512];
 };
 
-struct gfs_indirect {
-	struct gfs2_meta_header in_header;
-
-	char in_reserved[64];
-};
-
 struct blkstack_info {
 	uint64_t block;
 	int start_row[DMODES];
@@ -195,10 +191,7 @@ struct blkstack_info {
 	int edit_col[DMODES];
 	enum dsp_mode dmode;
 	int gfs2_struct_type;
-};
-
-struct metapath {
-	uint64_t mp_list[GFS2_MAX_META_HEIGHT];
+	struct metapath mp;
 };
 
 struct gfs_sb {
@@ -230,28 +223,18 @@ struct gfs_sb {
 	char sb_reserved[96];
 };
 
-EXTERN struct blkstack_info blockstack[BLOCK_STACK_SIZE];
-EXTERN struct iinfo *indirect; /* more than the most indirect
+extern struct blkstack_info blockstack[BLOCK_STACK_SIZE];
+extern struct iinfo *indirect; /* more than the most indirect
 			       pointers possible for any given 4K block */
-EXTERN struct indirect_info masterdir; /* Master directory info */
-EXTERN int indirect_blocks INIT(0);  /* count of indirect blocks */
-EXTERN enum dsp_mode dmode INIT(HEX_MODE);
+extern struct indirect_info masterdir; /* Master directory info */
+extern int indirect_blocks;  /* count of indirect blocks */
+extern enum dsp_mode dmode;
 
 #define SCREEN_HEIGHT   (16)
 #define SCREEN_WIDTH    (16)
 
 /*  Memory macros  */
 
-#define type_zalloc(ptr, type, count) \
-{ \
-  (ptr) = (type *)malloc(sizeof(type) * (count)); \
-  if ((ptr)) \
-    memset((char *)(ptr), 0, sizeof(type) * (count)); \
-  else \
-    die("unable to allocate memory on line %d of file %s\n", \
-	__LINE__, __FILE__); \
-}
-
 #define type_alloc(ptr, type, count) \
 { \
   (ptr) = (type *)malloc(sizeof(type) * (count)); \
@@ -266,7 +249,7 @@ EXTERN enum dsp_mode dmode INIT(HEX_MODE);
 #define DIV_RU(x, y) (((x) + (y) - 1) / (y))
 
 #define TITLE1 "gfs2_edit - Global File System Editor (use with extreme caution)"
-#define TITLE2 "Copyright (C) 2006 Red Hat, Inc. - Press H for help"
+#define TITLE2 REDHAT_COPYRIGHT " - Press H for help"
 
 #define COLOR_TITLE     1
 #define COLOR_NORMAL    2
