@@ -565,16 +565,16 @@ extern void gfs1_block_map(struct gfs2_inode *ip, uint64_t lblock, int *new,
 extern int gfs1_readi(struct gfs2_inode *ip, void *buf, uint64_t offset,
 		      unsigned int size);
 extern int gfs1_rindex_read(struct gfs2_sbd *sdp, int fd, int *count1);
-extern int gfs1_ri_update(struct gfs2_sbd *sdp, int fd, int *rgcount);
+extern int gfs1_ri_update(struct gfs2_sbd *sdp, int fd, int *rgcount, int quiet);
 extern struct gfs2_inode *gfs_inode_get(struct gfs2_sbd *sdp,
 					struct gfs2_buffer_head *bh);
 
 /* gfs2_log.c */
 struct gfs2_options {
 	char *device;
-	int yes:1;
-	int no:1;
-	int query:1;
+	unsigned int yes:1;
+	unsigned int no:1;
+	unsigned int query:1;
 };
 
 #define MSG_DEBUG       7
