@@ -23,24 +23,10 @@
  * Returns: -1 on error (with errno set), 0 on success (with @bytes set)
  */
 
-static int
-do_device_size(int fd, uint64_t *bytes)
+static int do_device_size(int fd, uint64_t *bytes)
 {
 	off_t off;
-#if 0
-	int error;
-	unsigned long size;
 
-	error = ioctl(fd, BLKGETSIZE64, bytes);	/* Size in bytes */
-	if (!error)
-		return 0;
-
-	error = ioctl(fd, BLKGETSIZE, &size);	/* Size in 512-byte blocks */
-	if (!error) {
-		*bytes = ((uint64_t) size) << 9;
-		return 0;
-	}
-#endif
 	off = lseek(fd, 0, SEEK_END);
 	if (off >= 0) {
 		*bytes = off;
@@ -58,8 +44,7 @@ do_device_size(int fd, uint64_t *bytes)
  * Returns: -1 on error (with errno set), 0 on success (with @bytes set)
  */
 
-int
-device_size(int fd, uint64_t *bytes)
+int device_size(int fd, uint64_t *bytes)
 {
 	struct stat st;
 	int error;
