@@ -46,6 +46,8 @@ struct node {
 	int fs_notified;
 	uint64_t add_time;
 	uint64_t fence_time;	/* for debug */
+	uint64_t cluster_add_time;
+	uint64_t cluster_remove_time;
 	uint32_t fence_queries;	/* for debug */
 	uint32_t added_seq;	/* for queries */
 	uint32_t removed_seq;	/* for queries */
@@ -71,6 +73,7 @@ struct change {
 	int we_joined;
 	uint32_t seq; /* used as a reference for debugging, and for queries */
 	uint32_t combined_seq; /* for queries */
+	uint64_t create_time;
 };
 
 struct ls_info {
@@ -91,13 +94,66 @@ struct id_info {
 };
 
 int message_flow_control_on;
-static int daemon_cpg_fd;
-static cpg_handle_t daemon_cpg_handle;
+static cpg_handle_t cpg_handle_daemon;
+static int cpg_fd_daemon;
 static struct protocol our_protocol;
 static struct list_head daemon_nodes;
 static struct cpg_address daemon_member[MAX_NODES];
 static int daemon_member_count;
 
+static void log_config(const struct cpg_name *group_name,
+		       const struct cpg_address *member_list,
+		       size_t member_list_entries,
+		       const struct cpg_address *left_list,
+		       size_t left_list_entries,
+		       const struct cpg_address *joined_list,
+		       size_t joined_list_entries)
+{
+	char m_buf[128];
+	char j_buf[32];
+	char l_buf[32];
+	size_t i, len, pos;
+	int ret;
+
+	memset(m_buf, 0, sizeof(m_buf));
+	memset(j_buf, 0, sizeof(j_buf));
+	memset(l_buf, 0, sizeof(l_buf));
+
+	len = sizeof(m_buf);
+	pos = 0;
+	for (i = 0; i < member_list_entries; i++) {
+		ret = snprintf(m_buf + pos, len - pos, " %d",
+			       member_list[i].nodeid);
+		if (ret >= len - pos)
+			break;
+		pos += ret;
+	}
+
+	len = sizeof(j_buf);
+	pos = 0;
+	for (i = 0; i < joined_list_entries; i++) {
+		ret = snprintf(j_buf + pos, len - pos, " %d",
+			       joined_list[i].nodeid);
+		if (ret >= len - pos)
+			break;
+		pos += ret;
+	}
+
+	len = sizeof(l_buf);
+	pos = 0;
+	for (i = 0; i < left_list_entries; i++) {
+		ret = snprintf(l_buf + pos, len - pos, " %d",
+			       left_list[i].nodeid);
+		if (ret >= len - pos)
+			break;
+		pos += ret;
+	}
+
+	log_debug("%s conf %zu %zu %zu memb%s join%s left%s", group_name->value,
+		  member_list_entries, joined_list_entries, left_list_entries,
+		  m_buf, j_buf, l_buf);
+}
+
 static void ls_info_in(struct ls_info *li)
 {
 	li->ls_info_size  = le32_to_cpu(li->ls_info_size);
@@ -359,7 +415,44 @@ static void node_history_init(struct lockspace *ls, int nodeid,
 	node->add_time = 0;
 	list_add_tail(&node->list, &ls->node_history);
  out:
-	node->added_seq = cg->seq;	/* for queries */
+	if (cg)
+		node->added_seq = cg->seq;	/* for queries */
+}
+
+void node_history_cluster_add(int nodeid)
+{
+	struct lockspace *ls;
+	struct node *node;
+
+	list_for_each_entry(ls, &lockspaces, list) {
+		node_history_init(ls, nodeid, NULL);
+
+		node = get_node_history(ls, nodeid);
+		if (!node) {
+			log_error("node_history_cluster_add no nodeid %d",
+				  nodeid);
+			return;
+		}
+
+		node->cluster_add_time = time(NULL);
+	}
+}
+
+void node_history_cluster_remove(int nodeid)
+{
+	struct lockspace *ls;
+	struct node *node;
+
+	list_for_each_entry(ls, &lockspaces, list) {
+		node = get_node_history(ls, nodeid);
+		if (!node) {
+			log_error("node_history_cluster_remove no nodeid %d",
+				  nodeid);
+			return;
+		}
+
+		node->cluster_remove_time = time(NULL);
+	}
 }
 
 static void node_history_start(struct lockspace *ls, int nodeid)
@@ -655,9 +748,15 @@ static int wait_conditions_done(struct lockspace *ls)
 		return 0;
 	}
 
-	/* even though fencing also waits for quorum, checking fencing isn't
-	   sufficient because we don't want to start new lockspaces in an
-	   inquorate cluster */
+	/* fencing waits for quorum, so we don't need to check quorum for any
+	   reasons related to safety or protection, so enable_quorum defaults
+	   to 0.  This does mean that lockspaces (and cluster fs's) can be
+	   started/enabled in an inquorate cluster if there are no outstanding
+	   fencing operations.  Some users or apps may want lockspaces/fs's to
+	   only be enabled in a quorate cluster; enable_quorum can be set to 1
+	   to get that behavior.  The main advantage of not waiting for quorum
+	   here is to allow lockspaces to be shut down (and cluster fs's
+	   unmounted) in an inquorate cluster. */
 
 	if (!check_quorum_done(ls)) {
 		poll_quorum++;
@@ -766,7 +865,7 @@ static void set_plock_ckpt_node(struct lockspace *ls)
 }
 
 static struct id_info *get_id_struct(struct id_info *ids, int count, int size,
-                                     int nodeid)
+				     int nodeid)
 {
 	struct id_info *id = ids;
 	int i;
@@ -787,6 +886,7 @@ static int match_change(struct lockspace *ls, struct change *cg,
 {
 	struct id_info *id;
 	struct member *memb;
+	struct node *node;
 	uint32_t seq = hd->msgdata;
 	int i, members_mismatch;
 
@@ -800,7 +900,7 @@ static int match_change(struct lockspace *ls, struct change *cg,
 	if (!id) {
 		log_group(ls, "match_change %d:%u skip %u we are not in members",
 			  hd->nodeid, seq, cg->seq);
-                return 0;
+		return 0;
 	}
 
 	memb = find_memb(cg, hd->nodeid);
@@ -810,6 +910,36 @@ static int match_change(struct lockspace *ls, struct change *cg,
 		return 0;
 	}
 
+	if (memb->start_flags & DLM_MFLG_NACK) {
+		log_group(ls, "match_change %d:%u skip %u is nacked",
+			  hd->nodeid, seq, cg->seq);
+		return 0;
+	}
+
+	if (memb->start && hd->type == DLM_MSG_START) {
+		log_group(ls, "match_change %d:%u skip %u already start",
+			  hd->nodeid, seq, cg->seq);
+		return 0;
+	}
+
+	/* a node's start can't match a change if the node joined the cluster
+	   more recently than the change was created */
+
+	node = get_node_history(ls, hd->nodeid);
+	if (!node) {
+		log_group(ls, "match_change %d:%u skip cg %u no node history",
+			  hd->nodeid, seq, cg->seq);
+		return 0;
+	}
+
+	if (node->cluster_add_time > cg->create_time) {
+		log_group(ls, "match_change %d:%u skip cg %u created %llu "
+			  "cluster add %llu", hd->nodeid, seq, cg->seq,
+			  (unsigned long long)cg->create_time,
+			  (unsigned long long)node->cluster_add_time);
+		return 0;
+	}
+
 	/* verify this is the right change by matching the counts
 	   and the nodeids of the current members */
 
@@ -837,6 +967,7 @@ static int match_change(struct lockspace *ls, struct change *cg,
 		}
 		id = (struct id_info *)((char *)id + li->id_info_size);
 	}
+
 	if (members_mismatch)
 		return 0;
 
@@ -924,7 +1055,7 @@ static void receive_start(struct lockspace *ls, struct dlm_header *hd, int len)
 
 	added = is_added(ls, hd->nodeid);
 
-	if (added && li->started_count) {
+	if (added && li->started_count && ls->started_count) {
 		log_error("receive_start %d:%u add node with started_count %u",
 			  hd->nodeid, seq, li->started_count);
 
@@ -933,6 +1064,11 @@ static void receive_start(struct lockspace *ls, struct dlm_header *hd, int len)
 		return;
 	}
 
+	if (memb->start_flags & DLM_MFLG_NACK) {
+		log_group(ls, "receive_start %d:%u is NACK", hd->nodeid, seq);
+		return;
+	}
+
 	node_history_start(ls, hd->nodeid);
 	memb->start = 1;
 }
@@ -976,9 +1112,9 @@ static void receive_plocks_stored(struct lockspace *ls, struct dlm_header *hd,
 	ls->save_plocks = 0;
 }
 
-static void send_info(struct lockspace *ls, int type)
+static void send_info(struct lockspace *ls, struct change *cg, int type,
+		      uint32_t flags)
 {
-	struct change *cg;
 	struct dlm_header *hd;
 	struct ls_info *li;
 	struct id_info *id;
@@ -986,8 +1122,6 @@ static void send_info(struct lockspace *ls, int type)
 	char *buf;
 	int len, id_count;
 
-	cg = list_first_entry(&ls->changes, struct change, list);
-
 	id_count = cg->member_count;
 
 	len = sizeof(struct dlm_header) + sizeof(struct ls_info) +
@@ -1008,6 +1142,8 @@ static void send_info(struct lockspace *ls, int type)
 
 	hd->type = type;
 	hd->msgdata = cg->seq;
+	hd->flags = flags;
+
 	if (ls->joining)
 		hd->flags |= DLM_MFLG_JOINING;
 	if (!ls->need_plocks)
@@ -1043,12 +1179,45 @@ static void send_info(struct lockspace *ls, int type)
 
 static void send_start(struct lockspace *ls)
 {
-	send_info(ls, DLM_MSG_START);
+	struct change *cg = list_first_entry(&ls->changes, struct change, list);
+
+	send_info(ls, cg, DLM_MSG_START, 0);
 }
 
 static void send_plocks_stored(struct lockspace *ls)
 {
-	send_info(ls, DLM_MSG_PLOCKS_STORED);
+	struct change *cg = list_first_entry(&ls->changes, struct change, list);
+
+	send_info(ls, cg, DLM_MSG_PLOCKS_STORED, 0);
+}
+
+static int same_members(struct change *cg1, struct change *cg2)
+{
+	struct member *memb;
+
+	list_for_each_entry(memb, &cg1->members, list) {
+		if (!find_memb(cg2, memb->nodeid))
+			return 0;
+	}
+	return 1;
+}
+
+static void send_nacks(struct lockspace *ls, struct change *startcg)
+{
+	struct change *cg;
+
+	list_for_each_entry(cg, &ls->changes, list) {
+		if (cg->seq < startcg->seq &&
+		    cg->member_count == startcg->member_count &&
+		    cg->joined_count == startcg->joined_count &&
+		    cg->remove_count == startcg->remove_count &&
+		    cg->failed_count == startcg->failed_count &&
+		    same_members(cg, startcg)) {
+			log_group(ls, "send nack old cg %u new cg %u",
+				   cg->seq, startcg->seq);
+			send_info(ls, cg, DLM_MSG_START, DLM_MFLG_NACK);
+		}
+	}
 }
 
 static int nodes_added(struct lockspace *ls)
@@ -1141,6 +1310,7 @@ static void apply_changes(struct lockspace *ls)
 
 	case CGST_WAIT_CONDITIONS:
 		if (wait_conditions_done(ls)) {
+			send_nacks(ls, cg);
 			send_start(ls);
 			cg->state = CGST_WAIT_MESSAGES;
 		}
@@ -1166,6 +1336,7 @@ void process_lockspace_changes(void)
 	poll_fencing = 0;
 	poll_quorum = 0;
 	poll_fs = 0;
+
 	list_for_each_entry_safe(ls, safe, &lockspaces, list) {
 		if (!list_empty(&ls->changes))
 			apply_changes(ls);
@@ -1192,6 +1363,7 @@ static int add_change(struct lockspace *ls,
 	INIT_LIST_HEAD(&cg->members);
 	INIT_LIST_HEAD(&cg->removed);
 	cg->state = CGST_WAIT_CONDITIONS;
+	cg->create_time = time(NULL);
 	cg->seq = ++ls->change_seq;
 	if (!cg->seq)
 		cg->seq = ++ls->change_seq;
@@ -1275,7 +1447,8 @@ static int add_change(struct lockspace *ls,
 	return error;
 }
 
-static int we_left(const struct cpg_address *left_list, size_t left_list_entries)
+static int we_left(const struct cpg_address *left_list,
+		   size_t left_list_entries)
 {
 	int i;
 
@@ -1300,6 +1473,10 @@ static void confchg_cb(cpg_handle_t handle,
 	struct member *memb;
 	int rv;
 
+	log_config(group_name, member_list, member_list_entries,
+		   left_list, left_list_entries,
+		   joined_list, joined_list_entries);
+
 	ls = find_ls_handle(handle);
 	if (!ls) {
 		log_error("confchg_cb no lockspace for cpg %s",
@@ -1450,7 +1627,7 @@ void update_flow_control_status(void)
 	cpg_flow_control_state_t flow_control_state;
 	cpg_error_t error;
 
-	error = cpg_flow_control_state_get(daemon_cpg_handle,
+	error = cpg_flow_control_state_get(cpg_handle_daemon,
 					   &flow_control_state);
 	if (error != CPG_OK) {
 		log_error("cpg_flow_control_state_get %d", error);
@@ -1470,7 +1647,7 @@ void update_flow_control_status(void)
 	}
 }
 
-static void process_lockspace_cpg(int ci)
+static void process_cpg_lockspace(int ci)
 {
 	struct lockspace *ls;
 	cpg_error_t error;
@@ -1516,7 +1693,7 @@ int dlm_join_lockspace(struct lockspace *ls)
 
 	cpg_fd_get(h, &fd);
 
-	ci = client_add(fd, process_lockspace_cpg, NULL);
+	ci = client_add(fd, process_cpg_lockspace, NULL);
 
 	list_add(&ls->list, &lockspaces);
 
@@ -1846,7 +2023,7 @@ static void send_protocol(struct protocol *proto)
 	memcpy(pr, proto, sizeof(struct protocol));
 	protocol_out(pr);
 
-	_send_message(daemon_cpg_handle, buf, len, DLM_MSG_PROTOCOL);
+	_send_message(cpg_handle_daemon, buf, len, DLM_MSG_PROTOCOL);
 }
 
 int set_protocol(void)
@@ -1857,7 +2034,7 @@ int set_protocol(void)
 	int rv;
 
 	memset(&pollfd, 0, sizeof(pollfd));
-	pollfd.fd = daemon_cpg_fd;
+	pollfd.fd = cpg_fd_daemon;
 	pollfd.events = POLLIN;
 
 	while (1) {
@@ -1903,7 +2080,7 @@ int set_protocol(void)
 		}
 
 		if (pollfd.revents & POLLIN)
-			process_cpg(0);
+			process_cpg_daemon(0);
 		if (pollfd.revents & (POLLERR | POLLHUP | POLLNVAL)) {
 			log_error("set_protocol poll revents %u",
 				  pollfd.revents);
@@ -1949,6 +2126,8 @@ int set_protocol(void)
 		  our_protocol.kernel_max[0],
 		  our_protocol.kernel_max[1],
 		  our_protocol.kernel_max[2]);
+
+	send_protocol(&our_protocol);
 	return 0;
 }
 
@@ -1987,6 +2166,10 @@ static void confchg_cb_daemon(cpg_handle_t handle,
 {
 	int i;
 
+	log_config(group_name, member_list, member_list_entries,
+		   left_list, left_list_entries,
+		   joined_list, joined_list_entries);
+
 	if (joined_list_entries)
 		send_protocol(&our_protocol);
 
@@ -2004,16 +2187,16 @@ static cpg_callbacks_t cpg_callbacks_daemon = {
 	.cpg_confchg_fn = confchg_cb_daemon,
 };
 
-void process_cpg(int ci)
+void process_cpg_daemon(int ci)
 {
 	cpg_error_t error;
 
-	error = cpg_dispatch(daemon_cpg_handle, CPG_DISPATCH_ALL);
+	error = cpg_dispatch(cpg_handle_daemon, CPG_DISPATCH_ALL);
 	if (error != CPG_OK)
 		log_error("daemon cpg_dispatch error %d", error);
 }
 
-int setup_cpg(void)
+int setup_cpg_daemon(void)
 {
 	cpg_error_t error;
 	struct cpg_name name;
@@ -2029,20 +2212,20 @@ int setup_cpg(void)
 	our_protocol.kernel_max[1] = 1;
 	our_protocol.kernel_max[2] = 1;
 
-	error = cpg_initialize(&daemon_cpg_handle, &cpg_callbacks_daemon);
+	error = cpg_initialize(&cpg_handle_daemon, &cpg_callbacks_daemon);
 	if (error != CPG_OK) {
 		log_error("daemon cpg_initialize error %d", error);
 		return -1;
 	}
 
-	cpg_fd_get(daemon_cpg_handle, &daemon_cpg_fd);
+	cpg_fd_get(cpg_handle_daemon, &cpg_fd_daemon);
 
 	memset(&name, 0, sizeof(name));
 	sprintf(name.value, "dlm:controld");
 	name.length = strlen(name.value) + 1;
 
  retry:
-	error = cpg_join(daemon_cpg_handle, &name);
+	error = cpg_join(cpg_handle_daemon, &name);
 	if (error == CPG_ERR_TRY_AGAIN) {
 		sleep(1);
 		if (!(++i % 10))
@@ -2054,22 +2237,22 @@ int setup_cpg(void)
 		goto fail;
 	}
 
-	log_debug("setup_cpg %d", daemon_cpg_fd);
-	return daemon_cpg_fd;
+	log_debug("setup_cpg_daemon %d", cpg_fd_daemon);
+	return cpg_fd_daemon;
 
  fail:
-	cpg_finalize(daemon_cpg_handle);
+	cpg_finalize(cpg_handle_daemon);
 	return -1;
 }
 
-void close_cpg(void)
+void close_cpg_daemon(void)
 {
 	struct lockspace *ls;
 	cpg_error_t error;
 	struct cpg_name name;
 	int i = 0;
 
-	if (!daemon_cpg_handle)
+	if (!cpg_handle_daemon)
 		return;
 	if (cluster_down)
 		goto fin;
@@ -2079,7 +2262,7 @@ void close_cpg(void)
 	name.length = strlen(name.value) + 1;
 
  retry:
-	error = cpg_leave(daemon_cpg_handle, &name);
+	error = cpg_leave(cpg_handle_daemon, &name);
 	if (error == CPG_ERR_TRY_AGAIN) {
 		sleep(1);
 		if (!(++i % 10))
@@ -2093,7 +2276,7 @@ void close_cpg(void)
 		if (ls->cpg_handle)
 			cpg_finalize(ls->cpg_handle);
 	}
-	cpg_finalize(daemon_cpg_handle);
+	cpg_finalize(cpg_handle_daemon);
 }
 
 /* fs_controld has seen nodedown for nodeid; it's now ok for dlm to do
