@@ -73,6 +73,7 @@ extern int poll_fencing;
 extern int poll_quorum;
 extern int poll_fs;
 extern int poll_ignore_plock;
+extern int poll_drop_plock;
 extern int plock_fd;
 extern int plock_ci;
 extern struct list_head lockspaces;
@@ -109,7 +110,7 @@ do { \
 	snprintf(daemon_debug_buf, 255, "%ld %s " fmt "\n", time(NULL), \
 		 (ls)->name, ##args); \
 	daemon_dump_save(); \
-	logt_print(LOG_DEBUG, fmt "\n", ##args); \
+	logt_print(LOG_DEBUG, "%s " fmt "\n", (ls)->name, ##args); \
 	if (daemon_debug_opt) \
 		fprintf(stderr, "%s", daemon_debug_buf); \
 } while (0)
@@ -141,6 +142,8 @@ enum {
 /* dlm_header flags */
 #define DLM_MFLG_JOINING   1  /* accompanies start, we are joining */
 #define DLM_MFLG_HAVEPLOCK 2  /* accompanies start, we have plock state */
+#define DLM_MFLG_NACK      4  /* accompanies start, prevent wrong match when
+				 two outstanding changes are the same */
 
 struct dlm_header {
 	uint16_t version[3];
@@ -231,9 +234,9 @@ int read_ccs_int(const char *path, int *config_val);
 int get_weight(int nodeid, char *lockspace);
 
 /* cpg.c */
-int setup_cpg(void);
-void close_cpg(void);
-void process_cpg(int ci);
+int setup_cpg_daemon(void);
+void close_cpg_daemon(void);
+void process_cpg_daemon(int ci);
 int set_protocol(void);
 void process_lockspace_changes(void);
 void dlm_send_message(struct lockspace *ls, char *buf, int len);
@@ -241,6 +244,8 @@ int dlm_join_lockspace(struct lockspace *ls);
 int dlm_leave_lockspace(struct lockspace *ls);
 const char *msg_name(int type);
 void update_flow_control_status(void);
+void node_history_cluster_add(int nodeid);
+void node_history_cluster_remove(int nodeid);
 int set_node_info(struct lockspace *ls, int nodeid, struct dlmc_node *node);
 int set_lockspace_info(struct lockspace *ls, struct dlmc_lockspace *lockspace);
 int set_lockspaces(int *count, struct dlmc_lockspace **lss_out);
@@ -296,6 +301,7 @@ void process_netlink(int ci);
 int setup_plocks(void);
 void close_plocks(void);
 void process_plocks(int ci);
+void drop_resources_all(void);
 int limit_plocks(void);
 void receive_plock(struct lockspace *ls, struct dlm_header *hd, int len);
 void receive_own(struct lockspace *ls, struct dlm_header *hd, int len);
