@@ -32,7 +32,7 @@ int do_read(int fd, void *buf, size_t count)
 	int rv, off = 0;
 
 	while (off < count) {
-		rv = read(fd, buf + off, count - off);
+		rv = read(fd, (char *)buf + off, count - off);
 		if (rv == 0)
 			return -1;
 		if (rv == -1 && errno == EINTR)
@@ -49,7 +49,7 @@ int do_write(int fd, void *buf, size_t count)
 	int rv, off = 0;
 
  retry:
-	rv = write(fd, buf + off, count);
+	rv = write(fd, (char *)buf + off, count);
 	if (rv == -1 && errno == EINTR)
 		goto retry;
 	if (rv < 0) {
@@ -268,7 +268,7 @@ static char *get_args(char *buf, int *argc, char **argv, char sep, int want)
 	return rp;
 }
 
-char *dlm_mode_str(int mode)
+const char *dlm_mode_str(int mode)
 {
 	switch (mode) {
 	case DLM_LOCK_IV:
@@ -304,12 +304,11 @@ static void process_uevent(int ci)
 
  retry_recv:
 	rv = recv(client[ci].fd, &buf, sizeof(buf), 0);
-	if (rv == -1 && rv == EINTR)
-		goto retry_recv;
-	if (rv == -1 && rv == EAGAIN)
-		return;
 	if (rv < 0) {
-		log_error("uevent recv error %d errno %d", rv, errno);
+		if (errno == EINTR)
+			goto retry_recv;
+		if (errno != EAGAIN)
+			log_error("uevent recv error %d errno %d", rv, errno);
 		return;
 	}
 
@@ -720,7 +719,7 @@ static void process_listener(int ci)
 	log_debug("client connection %d fd %d", i, fd);
 }
 
-static int setup_listener(char *sock_path)
+static int setup_listener(const char *sock_path)
 {
 	struct sockaddr_un addr;
 	socklen_t addrlen;
@@ -755,12 +754,12 @@ static int setup_listener(char *sock_path)
 	return s;
 }
 
-void query_lock(void)
+static void query_lock(void)
 {
 	pthread_mutex_lock(&query_mutex);
 }
 
-void query_unlock(void)
+static void query_unlock(void)
 {
 	pthread_mutex_unlock(&query_mutex);
 }
@@ -777,14 +776,14 @@ static void *process_queries(void *arg)
 
 	rv = setup_listener(DLMC_QUERY_SOCK_PATH);
 	if (rv < 0)
-		exit (-1);
+		return NULL;
 
 	s = rv;
 
 	for (;;) {
 		f = accept(s, NULL, NULL);
 		if (f < 0)
-			exit (-1);
+			return NULL;
 
 		rv = do_read(f, &h, sizeof(h));
 		if (rv < 0) {
@@ -860,7 +859,8 @@ static void setup_monitor(void)
 
 void cluster_dead(int ci)
 {
-	log_error("cluster is down, exiting");
+	if (!cluster_down)
+		log_error("cluster is down, exiting");
 	daemon_quit = 1;
 	cluster_down = 1;
 }
@@ -919,17 +919,29 @@ static void loop(void)
 			goto out;
 		client_add(rv, process_groupd, cluster_dead);
 
-		group_mode = GROUP_LIBGROUP;
-		if (cfgd_groupd_compat == 2)
-			set_group_mode();
+		switch (cfgd_groupd_compat) {
+		case 1:
+			group_mode = GROUP_LIBGROUP;
+			rv = 0;
+			break;
+		case 2:
+			rv = set_group_mode();
+			break;
+		default:
+			log_error("inval groupd_compat %d", cfgd_groupd_compat);
+			rv = -1;
+			break;
+		}
+		if (rv < 0) 
+			goto out;
 	}
 	log_debug("group_mode %d compat %d", group_mode, cfgd_groupd_compat);
 
 	if (group_mode == GROUP_LIBCPG) {
-		rv = setup_cpg();
+		rv = setup_cpg_daemon();
 		if (rv < 0)
 			goto out;
-		client_add(rv, process_cpg, cluster_dead);
+		client_add(rv, process_cpg_daemon, cluster_dead);
 
 		rv = set_protocol();
 		if (rv < 0)
@@ -999,13 +1011,22 @@ static void loop(void)
 			}
 			poll_timeout = 1000;
 		}
+
+		if (poll_drop_plock) {
+			drop_resources_all();
+			if (poll_drop_plock)
+				poll_timeout = 1000;
+		}
+
 		query_unlock();
 	}
  out:
 	if (cfgd_groupd_compat)
 		close_groupd();
-	if (group_mode == GROUP_LIBCPG)
-		close_cpg();
+	if (group_mode == GROUP_LIBCPG) {
+		close_cpg_daemon();
+		close_plocks();
+	}
 	clear_configfs();
 	close_logging();
 	close_ccs();
@@ -1065,8 +1086,8 @@ static void print_usage(void)
 	printf("\n");
 	printf("Options:\n");
 	printf("\n");
-	printf("  -D		Enable daemon debugging and don't fork\n");
-	printf("  -L <num>	Enable (1) or disable (0) debugging to logsys (default %d)\n", DEFAULT_DEBUG_LOGSYS);
+	printf("  -D		Enable debugging to stderr and don't fork\n");
+	printf("  -L		Enable debugging to log file\n");
 	printf("  -K		Enable kernel dlm debugging messages\n");
 	printf("  -g <num>	groupd compatibility mode, 0 off, 1 on, 2 detect\n");
 	printf("		0: use libcpg, no backward compat, best performance\n");
@@ -1097,7 +1118,7 @@ static void print_usage(void)
 	printf("  -V		Print program version information, then exit\n");
 }
 
-#define OPTION_STRING "L:DKg:f:q:d:p:Pl:o:t:c:a:hV"
+#define OPTION_STRING "LDKg:f:q:d:p:Pl:o:t:c:a:hV"
 
 static void read_arguments(int argc, char **argv)
 {
@@ -1118,8 +1139,8 @@ static void read_arguments(int argc, char **argv)
 			break;
 
 		case 'L':
-			optd_debug_logsys = 1;
-			cfgd_debug_logsys = atoi(optarg);
+			optd_debug_logfile = 1;
+			cfgd_debug_logfile = 1;
 			break;
 
 		case 'g':
@@ -1211,9 +1232,9 @@ static void read_arguments(int argc, char **argv)
 		};
 	}
 
-	if (!optd_debug_logsys && getenv("DLM_CONTROLD_DEBUG")) {
-		optd_debug_logsys = 1;
-		cfgd_debug_logsys = atoi(getenv("DLM_CONTROLD_DEBUG"));
+	if (getenv("DLM_CONTROLD_DEBUG")) {
+		optd_debug_logfile = 1;
+		cfgd_debug_logfile = 1;
 	}
 }
 
@@ -1253,7 +1274,6 @@ int main(int argc, char **argv)
 	INIT_LIST_HEAD(&fs_register_list);
 
 	read_arguments(argc, argv);
-	lockfile();
 
 	if (!daemon_debug_opt) {
 		if (daemon(0, 0) < 0) {
@@ -1261,13 +1281,16 @@ int main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
+	lockfile();
 	init_logging();
+	log_level(LOG_INFO, "dlm_controld %s started", RELEASE_VERSION);
 	signal(SIGTERM, sigterm_handler);
 	set_scheduler();
 	set_oom_adj(-16);
 
 	loop();
 
+	unlink(LOCKFILE_NAME);
 	return 0;
 }
 
@@ -1294,6 +1317,7 @@ int poll_fencing;
 int poll_quorum;
 int poll_fs;
 int poll_ignore_plock;
+int poll_drop_plock;
 int plock_fd;
 int plock_ci;
 struct list_head lockspaces;
@@ -1318,7 +1342,7 @@ int optk_debug;
 int optk_timewarn;
 int optk_protocol;
 int optd_groupd_compat;
-int optd_debug_logsys;
+int optd_debug_logfile;
 int optd_enable_fencing;
 int optd_enable_quorum;
 int optd_enable_deadlk;
@@ -1337,7 +1361,7 @@ int cfgk_debug                  = -1;
 int cfgk_timewarn               = -1;
 int cfgk_protocol               = -1;
 int cfgd_groupd_compat          = DEFAULT_GROUPD_COMPAT;
-int cfgd_debug_logsys           = DEFAULT_DEBUG_LOGSYS;
+int cfgd_debug_logfile		= DEFAULT_DEBUG_LOGFILE;
 int cfgd_enable_fencing         = DEFAULT_ENABLE_FENCING;
 int cfgd_enable_quorum          = DEFAULT_ENABLE_QUORUM;
 int cfgd_enable_deadlk          = DEFAULT_ENABLE_DEADLK;
