@@ -34,7 +34,7 @@
 #include <openais/saAis.h>
 #include <openais/saCkpt.h>
 #include <corosync/cpg.h>
-#include <corosync/engine/logsys.h>
+#include <liblogthread.h>
 
 #include <linux/dlmconstants.h>
 #include "libgfscontrol.h"
@@ -70,7 +70,7 @@ extern int poll_ignore_plock;
 extern int plock_fd;
 extern int plock_ci;
 extern struct list_head mountgroups;
-extern int cman_quorate;
+extern int cluster_quorate;
 extern int our_nodeid;
 extern char *clustername;
 extern char daemon_debug_buf[256];
@@ -83,35 +83,33 @@ extern int dmsetup_wait;
 extern cpg_handle_t cpg_handle_daemon;
 extern int libcpg_flow_control_on;
 extern int group_mode;
+extern uint32_t plock_minor;
+extern uint32_t old_plock_minor;
 extern struct list_head withdrawn_mounts;
+extern int using_default_plock_ownership;
 
 void daemon_dump_save(void);
 
-#define log_debug(fmt, args...) \
+#define log_level(lvl, fmt, args...) \
 do { \
 	snprintf(daemon_debug_buf, 255, "%ld " fmt "\n", time(NULL), ##args); \
 	daemon_dump_save(); \
+	logt_print(lvl, fmt "\n", ##args); \
 	if (daemon_debug_opt) \
 		fprintf(stderr, "%s", daemon_debug_buf); \
-	if (cfgd_debug_logsys) \
-		log_printf(LOG_DEBUG, "%s", daemon_debug_buf); \
 } while (0)
 
+#define log_debug(fmt, args...) log_level(LOG_DEBUG, fmt, ##args)
+#define log_error(fmt, args...) log_level(LOG_ERR, fmt, ##args)
+
 #define log_group(g, fmt, args...) \
 do { \
 	snprintf(daemon_debug_buf, 255, "%ld %s " fmt "\n", time(NULL), \
 		 (g)->name, ##args); \
 	daemon_dump_save(); \
+	logt_print(LOG_DEBUG, "%s " fmt "\n", (g)->name, ##args); \
 	if (daemon_debug_opt) \
 		fprintf(stderr, "%s", daemon_debug_buf); \
-	if (cfgd_debug_logsys) \
-		log_printf(LOG_DEBUG, "%s", daemon_debug_buf); \
-} while (0)
-
-#define log_error(fmt, args...) \
-do { \
-	log_debug(fmt, ##args); \
-	log_printf(LOG_ERR, fmt, ##args); \
 } while (0)
 
 #define log_plock(g, fmt, args...) \
@@ -161,6 +159,7 @@ struct mountgroup {
 	struct list_head        changes;
 	struct list_head        node_history;
 	struct list_head	journals;
+	int			dlm_registered;
 	int			dlm_notify_nodeid;
 	int			first_done_uevent;
 	int			first_recovery_needed;
@@ -216,22 +215,22 @@ struct mountgroup {
 /* config.c */
 int setup_ccs(void);
 void close_ccs(void);
-void read_ccs_name(char *path, char *name);
-void read_ccs_yesno(char *path, int *yes, int *no);
-void read_ccs_int(char *path, int *config_val);
+void read_ccs_name(const char *path, char *name);
+void read_ccs_yesno(const char *path, int *yes, int *no);
+int read_ccs_int(const char *path, int *config_val);
 void read_ccs_nodir(struct mountgroup *mg, char *buf);
 
 /* cpg-new.c */
-int setup_cpg(void);
-void close_cpg(void);
-void process_cpg(int ci);
+int setup_cpg_daemon(void);
+void close_cpg_daemon(void);
+void process_cpg_daemon(int ci);
 int setup_dlmcontrol(void);
 void process_dlmcontrol(int ci);
 int set_protocol(void);
-void process_recovery_uevent(char *table);
+void process_recovery_uevent(char *name, int jid, int status, int first);
 void process_mountgroups(void);
 int gfs_join_mountgroup(struct mountgroup *mg);
-void do_leave(char *table, int mnterr);
+void do_leave(char *name, int mnterr);
 void gfs_mount_done(struct mountgroup *mg);
 void send_remount(struct mountgroup *mg, struct gfsc_mount_args *ma);
 void send_withdraw(struct mountgroup *mg);
@@ -241,6 +240,8 @@ int set_mountgroups(int *count, struct gfsc_mountgroup **mgs_out);
 int set_mountgroup_nodes(struct mountgroup *mg, int option, int *node_count,
 	struct gfsc_node **nodes_out);
 void free_mg(struct mountgroup *mg);
+void node_history_cluster_add(int nodeid);
+void node_history_cluster_remove(int nodeid);
 
 /* cpg-old.c */
 int setup_cpg_old(void);
@@ -248,13 +249,12 @@ void close_cpg_old(void);
 void process_cpg_old(int ci);
 
 int gfs_join_mountgroup_old(struct mountgroup *mg, struct gfsc_mount_args *ma);
-void do_leave_old(char *table, int mnterr);
+void do_leave_old(char *name, int mnterr);
 int send_group_message_old(struct mountgroup *mg, int len, char *buf);
-void save_message_old(struct mountgroup *mg, char *buf, int len, int from,
+void save_message_old(struct mountgroup *mg, char *buf, size_t len, int from,
 	int type);
 void send_withdraw_old(struct mountgroup *mg);
-int process_recovery_uevent_old(char *table);
-void ping_kernel_mount_old(char *table);
+int process_recovery_uevent_old(char *name, int jid, int status, int first);
 void send_remount_old(struct mountgroup *mg, struct gfsc_mount_args *ma);
 void send_mount_status_old(struct mountgroup *mg);
 int do_stop(struct mountgroup *mg);
@@ -274,7 +274,7 @@ int set_node_info_group(struct mountgroup *mg, int nodeid,
 int set_mountgroups_group(int *count, struct gfsc_mountgroup **mgs_out);
 int set_mountgroup_nodes_group(struct mountgroup *mg, int option,
 	int *node_count, struct gfsc_node **nodes_out);
-void set_group_mode(void);
+int set_group_mode(void);
 
 /* main.c */
 int do_read(int fd, void *buf, size_t count);
@@ -296,13 +296,14 @@ void process_connection(int ci);
 void cluster_dead(int ci);
 
 /* member_cman.c */
-int setup_cman(void);
-void close_cman(void);
-void process_cman(int ci);
+int setup_cluster(void);
+void close_cluster(void);
+void process_cluster(int ci);
 void kick_node_from_cluster(int nodeid);
 
 /* plock.c */
 int setup_plocks(void);
+void close_plocks(void);
 void process_plocks(int ci);
 int limit_plocks(void);
 void receive_plock(struct mountgroup *mg, char *buf, int len, int from);
@@ -315,20 +316,23 @@ void store_plocks(struct mountgroup *mg, int nodeid);
 void retrieve_plocks(struct mountgroup *mg);
 void purge_plocks(struct mountgroup *mg, int nodeid, int unmount);
 int fill_plock_dump_buf(struct mountgroup *mg);
+int setup_misc_devices(void);
 
 /* util.c */
 int we_are_in_fence_domain(void);
-int set_sysfs(struct mountgroup *mg, char *field, int val);
-int read_sysfs_int(struct mountgroup *mg, char *field, int *val_out);
+int set_sysfs(struct mountgroup *mg, const char *field, int val);
+int read_sysfs_int(struct mountgroup *mg, const char *field, int *val_out);
 int run_dmsetup_suspend(struct mountgroup *mg, char *dev);
 void update_dmsetup_wait(void);
 void update_flow_control_status(void);
 int check_uncontrolled_filesystems(void);
 
 /* logging.c */
-
 void init_logging(void);
-void setup_logging();
+void setup_logging(void);
 void close_logging(void);
 
+/* crc.c */
+uint32_t cpgname_to_crc(const char *data, int len);
+
 #endif
