@@ -30,7 +30,7 @@ int do_read(int fd, void *buf, size_t count)
 	int rv, off = 0;
 
 	while (off < count) {
-		rv = read(fd, buf + off, count - off);
+		rv = read(fd, (char *)buf + off, count - off);
 		if (rv == 0)
 			return -1;
 		if (rv == -1 && errno == EINTR)
@@ -47,7 +47,7 @@ int do_write(int fd, void *buf, size_t count)
 	int rv, off = 0;
 
  retry:
-	rv = write(fd, buf + off, count);
+	rv = write(fd, (char *)buf + off, count);
 	if (rv == -1 && errno == EINTR)
 		goto retry;
 	if (rv < 0) {
@@ -984,7 +984,7 @@ static void process_listener(int ci)
 	log_debug("client connection %d fd %d", i, fd);
 }
 
-static int setup_listener(char *sock_path)
+static int setup_listener(const char *sock_path)
 {
 	struct sockaddr_un addr;
 	socklen_t addrlen;
@@ -1040,14 +1040,14 @@ static void *process_queries(void *arg)
 
 	rv = setup_listener(GFSC_QUERY_SOCK_PATH);
 	if (rv < 0)
-		exit (-1);
+		return NULL;
 
 	s = rv;
 
 	for (;;) {
 		f = accept(s, NULL, NULL);
 		if (f < 0)
-			exit (-1);
+			return NULL;
 
 		rv = do_read(f, &h, sizeof(h));
 		if (rv < 0) {
@@ -1115,13 +1115,6 @@ void cluster_dead(int ci)
 	cluster_down = 1;
 }
 
-static void dlmcontrol_dead(int ci)
-{
-	if (!list_empty(&mountgroups))
-		log_error("dlm_controld is gone");
-	client_dead(ci);
-}
-
 static void loop(void)
 {
 	int poll_timeout = -1;
@@ -1138,10 +1131,10 @@ static void loop(void)
 		goto out;
 	client_add(rv, process_listener, NULL);
 
-	rv = setup_cman();
+	rv = setup_cluster();
 	if (rv < 0)
 		goto out;
-	client_add(rv, process_cman, cluster_dead);
+	client_add(rv, process_cluster, cluster_dead);
 
 	rv = setup_ccs();
 	if (rv < 0)
@@ -1191,10 +1184,10 @@ static void loop(void)
 		 * code in: cpg-new.c
 		 */
 
-		rv = setup_cpg();
+		rv = setup_cpg_daemon();
 		if (rv < 0)
 			goto out;
-		client_add(rv, process_cpg, cluster_dead);
+		client_add(rv, process_cpg_daemon, cluster_dead);
 
 		rv = set_protocol();
 		if (rv < 0)
@@ -1203,10 +1196,15 @@ static void loop(void)
 		rv = setup_dlmcontrol();
 		if (rv < 0)
 			goto out;
-		client_add(rv, process_dlmcontrol, dlmcontrol_dead);
+		client_add(rv, process_dlmcontrol, cluster_dead);
 
 	} else if (group_mode == GROUP_LIBGROUP) {
 
+		/* the stable2 default is zero plock_ownership which we need
+		   to match or protocol_active will differ */
+		if (using_default_plock_ownership)
+			cfgd_plock_ownership = 0;
+
 		/*
 		 * The old, bad, way of doing things using libgroup.
 		 * code in: cpg-old.c group.c plock.c
@@ -1293,14 +1291,16 @@ static void loop(void)
 	}
  out:
 	if (group_mode == GROUP_LIBCPG)
-		close_cpg();
-	else if (group_mode == GROUP_LIBGROUP)
+		close_cpg_daemon();
+	else if (group_mode == GROUP_LIBGROUP) {
+		close_plocks();
 		close_cpg_old();
+	}
 	if (cfgd_groupd_compat)
 		close_groupd();
 	close_logging();
 	close_ccs();
-	close_cman();
+	close_cluster();
 
 	if (!list_empty(&mountgroups))
 		log_error("mountgroups abandoned");
@@ -1521,7 +1521,6 @@ int main(int argc, char **argv)
 	INIT_LIST_HEAD(&withdrawn_mounts);
 
 	read_arguments(argc, argv);
-	lockfile();
 
 	if (!daemon_debug_opt) {
 		if (daemon(0, 0) < 0) {
@@ -1529,14 +1528,16 @@ int main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
+	lockfile();
 	init_logging();
-	log_level(LOG_INFO, "gfs_controld %s", RELEASE_VERSION);
+	log_level(LOG_INFO, "gfs_controld %s started", RELEASE_VERSION);
 	signal(SIGTERM, sigterm_handler);
 	set_scheduler();
 	set_oom_adj(-16);
 
 	loop();
 
+	unlink(LOCKFILE_NAME);
 	return 0;
 }
 
@@ -1564,7 +1565,7 @@ int poll_dlm;
 int plock_fd;
 int plock_ci;
 struct list_head mountgroups;
-int cman_quorate;
+int cluster_quorate;
 int our_nodeid;
 char *clustername;
 char daemon_debug_buf[256];
@@ -1580,4 +1581,5 @@ int group_mode;
 uint32_t plock_minor;
 uint32_t old_plock_minor;
 struct list_head withdrawn_mounts;
+int using_default_plock_ownership;
 
