@@ -20,17 +20,23 @@ void kick_node_from_cluster(int nodeid)
 
 static void cman_callback(cman_handle_t h, void *private, int reason, int arg)
 {
-	if (reason == CMAN_REASON_TRY_SHUTDOWN) {
+	switch (reason) {
+	case CMAN_REASON_TRY_SHUTDOWN:
 		if (list_empty(&mountgroups))
 			cman_replyto_shutdown(ch, 1);
 		else {
 			log_debug("no to cman shutdown");
 			cman_replyto_shutdown(ch, 0);
 		}
+		break;
+	case CMAN_REASON_CONFIG_UPDATE:
+		setup_logging();
+		setup_ccs();
+		break;
 	}
 }
 
-void process_cman(int ci)
+void process_cluster(int ci)
 {
 	int rv;
 
@@ -39,7 +45,7 @@ void process_cman(int ci)
 		cluster_dead(0);
 }
 
-int setup_cman(void)
+int setup_cluster(void)
 {
 	cman_node_t node;
 	int rv, fd;
@@ -59,6 +65,7 @@ int setup_cman(void)
 	ch = cman_init(NULL);
 	if (!ch) {
 		log_error("cman_init error %d", errno);
+		cman_finish(ch_admin);
 		return -ENOTCONN;
 	}
 
@@ -71,6 +78,7 @@ int setup_cman(void)
 		}
 		log_error("cman_is_active error %d", errno);
 		cman_finish(ch);
+		cman_finish(ch_admin);
 		return -ENOTCONN;
 	}
 
@@ -78,6 +86,7 @@ int setup_cman(void)
 	if (rv < 0) {
 		log_error("cman_start_notification error %d %d", rv, errno);
 		cman_finish(ch);
+		cman_finish(ch_admin);
 		return rv;
 	}
 
@@ -90,6 +99,7 @@ int setup_cman(void)
 		log_error("cman_get_cluster error %d %d", rv, errno);
 		cman_stop_notification(ch);
 		cman_finish(ch);
+		cman_finish(ch_admin);
 		return rv;
 	}
 	clustername = cluster.ci_name;
@@ -100,6 +110,7 @@ int setup_cman(void)
 		log_error("cman_get_node error %d %d", rv, errno);
 		cman_stop_notification(ch);
 		cman_finish(ch);
+		cman_finish(ch_admin);
 		fd = rv;
 		goto out;
 	}
@@ -108,8 +119,9 @@ int setup_cman(void)
 	return fd;
 }
 
-void close_cman(void)
+void close_cluster(void)
 {
 	cman_finish(ch);
+	cman_finish(ch_admin);
 }
 
