@@ -1,4 +1,3 @@
-#include <message.h>
 #include <platform.h>
 #include <ccs.h>
 #include <stdio.h>
@@ -7,54 +6,43 @@
 #include <unistd.h>
 #include <rg_locks.h>
 #include <fcntl.h>
+#include <restart_counter.h>
 #include <resgroup.h>
+#include <reslist.h>
 #include <logging.h>
 #include <members.h>
 #include <msgsimple.h>
 #include <vf.h>
 #include <lock.h>
+#include <message.h>
 #include <rg_queue.h>
 #include <malloc.h>
 #include <cman-private.h>
 #include <event.h>
 #include <members.h>
-
-#define L_SHUTDOWN (1<<2)
-#define L_SYS (1<<1)
-#define L_USER (1<<0)
+#include <daemon_init.h>
+#include <groups.h>
 
 #ifdef WRAP_THREADS
 void dump_thread_states(FILE *);
 #endif
-int configure_rgmanager(int ccsfd, int debug);
+static int configure_rgmanager(int ccsfd, int debug);
 void set_transition_throttling(int);
 
-void node_event(int, int, int, int);
-void node_event_q(int, int, int, int);
-int daemon_init(char *);
-void kill_resource_groups(void);
-void set_my_id(int);
 void flag_shutdown(int sig);
-void hard_exit(void);
-int send_rg_states(msgctx_t *, int);
-int svc_exists(char *);
-int watchdog_init(void);
-int32_t master_event_callback(char *key, uint64_t viewno, void *data, uint32_t datalen);
 
-int node_has_fencing(int nodeid);
-int fence_domain_joined(void);
+int watchdog_init(void);
+
 
 int shutdown_pending = 0, running = 1, need_reconfigure = 0;
 char debug = 0; /* XXX* */
 static int signalled = 0;
 static uint8_t ALIGNED port = RG_PORT;
-static char *rgmanager_lsname = "rgmanager"; /* XXX default */
+static char *rgmanager_lsname = (char *)"rgmanager"; /* XXX default */
 static int status_poll_interval = DEFAULT_CHECK_INTERVAL;
 
-int next_node_id(cluster_member_list_t *membership, int me);
-void malloc_dump_table(FILE *, size_t, size_t);
 
-void
+static void
 segfault(int __attribute__ ((unused)) sig)
 {
 	char ow[64];
@@ -68,7 +56,7 @@ segfault(int __attribute__ ((unused)) sig)
 }
 
 
-int
+static int
 send_exit_msg(msgctx_t *ctx)
 {
 	msg_send_simple(ctx, RG_EXITING, my_id(), 0);
@@ -77,7 +65,7 @@ send_exit_msg(msgctx_t *ctx)
 }
 
 
-void
+static void
 send_node_states(msgctx_t *ctx)
 {
 	int x;
@@ -103,13 +91,6 @@ send_node_states(msgctx_t *ctx)
 }
 
 
-void
-flag_reconfigure(int __attribute__ ((unused)) sig)
-{
-	need_reconfigure++;
-}
-
-
 /**
   This updates our local membership view and handles whether or not we
   should exit, as well as determines node transitions (thus, calling
@@ -118,7 +99,7 @@ flag_reconfigure(int __attribute__ ((unused)) sig)
   @see				node_event
   @return			0
  */
-int
+static int
 membership_update(void)
 {
 	cluster_member_list_t *new_ml = NULL, *node_delta = NULL,
@@ -280,7 +261,7 @@ membership_update(void)
 }
 
 
-int
+static int
 lock_commit_cb(char __attribute__ ((unused)) *key,
 	       uint64_t __attribute__ ((unused)) viewno,
 	       void *data, uint32_t datalen)
@@ -315,59 +296,7 @@ lock_commit_cb(char __attribute__ ((unused)) *key,
 }
 
 
-#if 0
-struct lr_arg {
-	msgctx_t *ctx;
-	int req;
-};
-
-
-void *
-lockreq_th(void *a)
-{
-	int ret;
-	char state;
-	struct lr_arg *lr_arg = (struct lr_arg *)a;
-	cluster_member_list_t *m = member_list();
-
-	state = (lr_arg->req==RG_LOCK)?1:0;
-	ret = vf_write(m, VFF_IGN_CONN_ERRORS, "rg_lockdown", &state, 1);
-	free_member_list(m);
-
-	if (ret == 0) {
-		msg_send_simple(lr_arg->ctx, RG_SUCCESS, 0, 0);
-	} else {
-		msg_send_simple(lr_arg->ctx, RG_FAIL, 0, 0);
-	}
-
-	msg_close(lr_arg->ctx);
-	msg_free_ctx(lr_arg->ctx);
-	free(lr_arg);
-	return NULL;
-}
-
-
-void
-do_lockreq(msgctx_t *ctx, int req)
-{
-	pthread_t th;
-	struct lr_arg *arg;
-
-	arg = malloc(sizeof(*arg));
-	if (!arg) {
-		msg_send_simple(ctx, RG_FAIL, 0, 0);
-		msg_close(ctx);
-		msg_free_ctx(ctx);
-		return 0;
-	}
-
-	arg->ctx = ctx;
-	arg->req = req;
-
-	pthread_create(&th, NULL, lockreq_th, (void *)arg);
-}
-#else
-void 
+static void 
 do_lockreq(msgctx_t *ctx, int req)
 {
 	int ret;
@@ -394,8 +323,6 @@ do_lockreq(msgctx_t *ctx, int req)
 		msg_send_simple(ctx, RG_FAIL, 0, 0);
 	}
 }
-#endif
-
 
 
 /**
@@ -407,7 +334,7 @@ do_lockreq(msgctx_t *ctx, int req)
  *			data received.  0 - handled message successfully.
  * @see			quorum_msg
  */
-int
+static int
 dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 {
 	int ret = 0, sz = -1, nid;
@@ -531,11 +458,20 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 
 		/* Distributed processing and/or request is from master node
 		   -- Queue request */
-		rt_enqueue_request(msg_sm->sm_data.d_svcName,
-		  		   msg_sm->sm_data.d_action,
-		  		   ctx, 0, msg_sm->sm_data.d_svcOwner,
-		  		   msg_sm->sm_hdr.gh_arg1,
-		  		   msg_sm->sm_hdr.gh_arg2);
+		if (rt_enqueue_request(msg_sm->sm_data.d_svcName,
+		      		       msg_sm->sm_data.d_action,
+		      		       ctx, 0, msg_sm->sm_data.d_svcOwner,
+		      		       msg_sm->sm_hdr.gh_arg1,
+		      		       msg_sm->sm_hdr.gh_arg2) != 0) {
+
+			/* Clean up this context if we fail to 
+			 * queue the request. */
+			send_ret(ctx, msg_sm->sm_data.d_svcName,
+				 RG_EAGAIN, msg_sm->sm_data.d_action, 0);
+			need_close = 1;
+			ret = 0;
+			goto out;
+		}
 		return 0;
 
 	case RG_EVENT:
@@ -596,7 +532,7 @@ out:
   @param fd		File descriptor to check
   @return		Event
  */
-int
+static int
 handle_cluster_event(msgctx_t *ctx)
 {
 	int ret;
@@ -677,8 +613,8 @@ void dump_config_version(FILE *fp);
 void dump_vf_states(FILE *fp);
 void dump_cluster_ctx(FILE *fp);
 
-void
-dump_internal_state(char *loc)
+static void
+dump_internal_state(const char *loc)
 {
 	FILE *fp;
 	fp=fopen(loc, "w+");
@@ -689,11 +625,10 @@ dump_internal_state(char *loc)
 	dump_thread_states(fp);
 #endif
 	dump_cluster_ctx(fp);
-	//malloc_dump_table(fp, 1, 16384); /* Only works if alloc.c us used */
  	fclose(fp);
 }
 
-int
+static int
 event_loop(msgctx_t *localctx, msgctx_t *clusterctx)
 {
  	int n = 0, max, ret;
@@ -795,17 +730,7 @@ flag_shutdown(int __attribute__ ((unused)) sig)
 }
 
 
-void
-hard_exit(void)
-{
-	rg_lockall(L_SYS);
-	rg_doall(RG_INIT, 1, "Emergency stop of %s");
-	//vf_shutdown();
-	exit(1);
-}
-
-
-void
+static void
 cleanup(msgctx_t *clusterctx)
 {
 	kill_resource_groups();
@@ -814,24 +739,22 @@ cleanup(msgctx_t *clusterctx)
 
 
 
-void
+static void
 statedump(int __attribute__ ((unused)) sig)
 {
 	signalled++;
 }
 
 
-void malloc_dump_table(FILE *, size_t, size_t);
-
-
 /*
  * Configure logging based on data in cluster.conf
  */
-int
+static int
 configure_rgmanager(int ccsfd, int dbg)
 {
 	char *v;
 	char internal = 0;
+	int status_child_max = 0;
 
 	if (ccsfd < 0) {
 		internal = 1;
@@ -864,12 +787,26 @@ configure_rgmanager(int ccsfd, int dbg)
 		} else {
 			logt_print(LOG_WARNING, "Ignoring illegal "
 			       "status_poll_interval of %s\n", v);
-			status_poll_interval = 10;
+			status_poll_interval = DEFAULT_CHECK_INTERVAL;
 		}
 		
 		free(v);
 	}
 
+	if (ccs_get(ccsfd, "/cluster/rm/@status_child_max", &v) == 0) {
+		status_child_max = atoi(v);
+		if (status_child_max >= 1) {
+			logt_print(LOG_NOTICE,
+			       "Status Child Max set to %d\n",
+			       status_child_max);
+			rg_set_childmax(status_child_max);
+		} else {
+			logt_print(LOG_WARNING, "Ignoring illegal "
+			       "status_child_max of %s\n", v);
+		}
+		
+		free(v);
+	}
 
 	if (internal)
 		ccs_disconnect(ccsfd);
@@ -878,8 +815,8 @@ configure_rgmanager(int ccsfd, int dbg)
 }
 
 
-void
-clu_initialize(cman_handle_t *ch)
+static int
+cman_connect(cman_handle_t *ch)
 {
 	if (!ch)
 		exit(1);
@@ -890,6 +827,8 @@ clu_initialize(cman_handle_t *ch)
 
 		while (!(*ch = cman_init(NULL))) {
 			sleep(1);
+			if (shutdown_pending)
+				return 1;
 		}
 	}
 
@@ -904,41 +843,39 @@ clu_initialize(cman_handle_t *ch)
 
 		while (cman_is_quorate(*ch) == 0) {
 			sleep(1);
+			if (shutdown_pending)
+				return 1;
 		}
 		logt_print(LOG_NOTICE, "Quorum formed\n");
 	}
 
+	return 0;
 }
 
 
-void
+static int
 wait_for_fencing(void)
 {
         if (node_has_fencing(my_id()) && !fence_domain_joined()) {
 		logt_print(LOG_INFO, "Waiting for fence domain join operation "
 		       "to complete\n");
 
-		while (fence_domain_joined() == 0)
+		while (fence_domain_joined() == 0) {
+			if (shutdown_pending)
+				return 1;
 			sleep(1);
+		}
 		logt_print(LOG_INFO, "Fence domain joined\n");
 	} else {
 		logt_print(LOG_DEBUG, "Fence domain already joined "
 		       "or no fencing configured\n");
 	}
+
+	return 0;
 }
 
 
-void
-set_nonblock(int fd)
-{
-       int flags;
-
-       flags = fcntl(fd, F_GETFL, 0);
-       fcntl(fd, F_SETFL, flags | O_NONBLOCK);
-}
-
-
-void *
+static void *
 shutdown_thread(void __attribute__ ((unused)) *arg)
 {
 	rg_lockall(L_SYS|L_SHUTDOWN);
@@ -983,6 +920,10 @@ main(int argc, char **argv)
 		}
 	}
 
+	if (getenv("RGMANAGER_DEBUG")) {
+		debug = 1;
+	}
+
 	if (!foreground && (geteuid() == 0)) {
 		daemon_init(argv[0]);
 		if (wd && !debug && !watchdog_init())
@@ -1002,7 +943,9 @@ main(int argc, char **argv)
 	}
 
 	init_logging(NULL, foreground, (debug? LOG_DEBUG : SYSLOGLEVEL));
-	clu_initialize(&clu);
+	if (cman_connect(&clu) != 0)
+		goto out;	/* Clean exit if sigint/sigterm here */
+
 	if (cman_init_subsys(clu) < 0) {
 		perror("cman_init_subsys");
 		return -1;
@@ -1025,7 +968,8 @@ main(int argc, char **argv)
 
 	logt_print(LOG_INFO, "I am node #%d\n", my_id());
 
-	wait_for_fencing();
+	if (wait_for_fencing() != 0)
+		goto out;
 
 	/*
 	   We know we're quorate.  At this point, we need to
@@ -1039,6 +983,9 @@ main(int argc, char **argv)
 		return -1;
 	}
 
+	if (shutdown_pending)
+		goto out;
+
 	if (msg_listen(MSG_SOCKET, RGMGR_SOCK, me.cn_nodeid, &local_ctx) < 0) {
 		logt_print(LOG_CRIT,
 		       "#10: Couldn't set up cluster message system: %s\n",
@@ -1098,13 +1045,15 @@ main(int argc, char **argv)
 
 	if (rg_initialized())
 		cleanup(cluster_ctx);
-	logt_print(LOG_NOTICE, "Shutdown complete, exiting\n");
 	clu_lock_finished(rgmanager_lsname);
+
+out:
+	logt_print(LOG_NOTICE, "Shutdown complete, exiting\n");
 	cman_finish(clu);
 	
 	close_logging();
-	/*malloc_dump_table(); */ /* Only works if alloc.c us used */
 	/*malloc_stats();*/
 
+	daemon_cleanup();
 	exit(0);
 }
