@@ -3,10 +3,13 @@
 #include <sys/wait.h>
 #include <sys/reboot.h>
 #include <stdlib.h>
+#include <sys/mman.h>
 
 #include <signals.h>
 #include <logging.h>
 
+int watchdog_init(void);
+
 static pid_t child = 0;
 
 static void 
@@ -14,6 +17,8 @@ signal_handler(int signum)
 {
         kill(child, signum);
 }
+
+
 static void 
 redirect_signals(void)
 {
@@ -33,6 +38,7 @@ redirect_signals(void)
 	}
 }
 
+
 /**
  return watchdog's pid, or 0 on failure
 */
@@ -50,6 +56,7 @@ watchdog_init(void)
 		return parent;
 	
 	redirect_signals();
+	mlockall(MCL_CURRENT); /* shouldn't need MCL_FUTURE */
 	
 	while (1) {
 	        if (waitpid(child, &status, 0) <= 0)
@@ -60,20 +67,22 @@ watchdog_init(void)
 		
 		if (WIFSIGNALED(status)) {
 		        if (WTERMSIG(status) == SIGKILL) {
-				log_printf(LOG_CRIT, "Watchdog: Daemon killed, exiting\n");
-				raise(SIGKILL);
-				while(1) ;
+				/* Assume the admin did a 'killall' - it will
+				 * kill us within a couple of seconds.  If 
+				 * we are still alive after this sleep, it
+				 * could have been the OOM killer killing
+				 * rgmanager proper and we need to reboot.
+				 */
+				sleep(3);
 			}
-			else {
 #ifdef DEBUG
-			        log_printf(LOG_CRIT, "Watchdog: Daemon died, but not rebooting because DEBUG is set\n");
+		        logt_print(LOG_CRIT, "Watchdog: Daemon died, but not rebooting because DEBUG is set\n");
 #else
-				log_printf(LOG_CRIT, "Watchdog: Daemon died, rebooting...\n");
-				sync();
-			        reboot(RB_AUTOBOOT);
+			logt_print(LOG_CRIT, "Watchdog: Daemon died, rebooting...\n");
+			sync();
+		        reboot(RB_AUTOBOOT);
 #endif
-				exit(255);
-			}
+			exit(255);
 		}
 	}
 }
